"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = usePopable;

var _react = require("react");

/**
 * Imperatively manage the popover.
 *
 * ```jsx
 * const [ref, { show, hide }] = usePopable()
 *
 * return <Popable ref={ref} />
 * ```
 *
 * If you prefer, you can destructure as an object:
 *
 * ```js
 * const { ref, show, hide } = usePopable()
 * ```
 */
function usePopable() {
  const ref = (0, _react.useRef)(null);
  const handlers = (0, _react.useMemo)(() => ({
    show: () => {
      var _ref$current;

      return (_ref$current = ref.current) === null || _ref$current === void 0 ? void 0 : _ref$current.show();
    },
    hide: () => {
      var _ref$current2;

      return (_ref$current2 = ref.current) === null || _ref$current2 === void 0 ? void 0 : _ref$current2.hide();
    }
  }), []);
  const result = [ref, handlers];
  result.show = handlers.show;
  result.hide = handlers.hide;
  result.ref = ref;
  return result;
}
//# sourceMappingURL=index.js.map